#This code will produce the simulation results for continuous Scenario 1 with an unequal number of responses.
#reflected in outputs Figures 62-63 of Supplementary Materials.

library(mvtnorm)
library(matrixcalc)

RESULT1 = NULL
RESULT2 = NULL
RESULT3 = NULL


simulations = 1000

np = parameters = 2
n = 10                          #Number of independent subjects: 10, 20, 30, 40
ni_max = sizes = 5                    #Responses: 5, 10


if(ni_max == 5)
{
  Cluster_Size=matrix(ni_max,n,1)
  a=c(3,4,5)
  for(j in 1:n)
    Cluster_Size[j,1] = sample(a,1)
}

if(ni_max == 10)
{
  Cluster_Size=matrix(ni_max,n,1)
  a=c(6,7,8,9,10)
  for(j in 1:n)
    Cluster_Size[j,1] = sample(a,1)
}

Corr_parm = 0.2
Corr_structure="exchangeable"     #This is the true correlation structure.

Beta_true = matrix(0, parameters, 1)
Beta_true[1, 1] = 0           #Intercept
Beta_true[2, 1] = 0           #Continuous Covariate


#Now for simulations
for (j in 1:simulations)
{
  subjleff = as.matrix(rnorm(n, mean = 0, sd = sqrt(0.25))) #subject level random effects, bi
  
  
  #Now I need to make the design matrix into having a row for every observation, not just for each cluster.
  # Also, I will be putting in subject IDs into the design matrix just so I have them
  X = NULL
  time_tot = NULL
  
  for (i in 1:n)
  {
    a = sample(1:Cluster_Size[i, 1], Cluster_Size[i, 1])
    time_orig = as.matrix(sort(a, decreasing = FALSE))         #Observed time points in order
    time  = time_orig + (ni_max - Cluster_Size[i, 1])          #make times last time points.
    
    variance = as.matrix(rnorm(Cluster_Size[i, 1], mean = 0, sd = sqrt(0.8)))^2              

    cont = as.matrix(rnorm(Cluster_Size[i, 1], mean = 0, sd = 1))  #Continuous covariate
    
    Design = matrix(1, Cluster_Size[i, 1], (parameters + 1))            #Implicitly gives values of 1 in the design matrix for the intercept, beta0
    Design[, 2] = cont
    Design[, 3] = i                                                 #Subject ID
    
    Y = Design[, 1:2] %*% Beta_true + variance + subjleff[i, 1]
    
    Design = cbind(Design, Y)
    X = rbind(X, Design)
    
    time_tot = rbind(time_tot, time_orig)
  }  

  #Now setting up the input (for GEE function) data matrices
  
  id = as.matrix(X[, 3])   #Matrix of subject IDs
  y = as.matrix(X[, 4])    #Matrix of outcome values
  x = as.matrix(X[, 2])  #Matrix of covariate values
  time = as.matrix(time_tot) #Matrix of time points
  

  ##############################################################
  #GEE results using the working exchangeable correlation structure
  ##############################################################
  result2 = GEE(id, y, x, fam="gaussian", corstr="exchangeable", b=NULL, maxiter=200, tol=0.00000001, time)
  result2a = t(as.matrix(result2[1:23, 1]))  #Results with respect to beta0
  result2b = t(as.matrix(result2[1:23, 2]))  #Results with respect to beta1
  result2c = matrix(0, 1, 23)
  ICC_est = result2[25, 1]
  var_est = result2[26, 2]
  
  
  df_KC = t(as.matrix(result2[27, ]))
  df_MD = t(as.matrix(result2[28, ]))
  df_AVG = t(as.matrix(result2[30, ]))
  df_WAVG = t(as.matrix(result2[31, ]))
  df_FG = t(as.matrix(result2[32, ]))
  df_GSH = t(as.matrix(result2[33, ]))
  df_FZZ = t(as.matrix(result2[34, ]))
  df_MBN = t(as.matrix(result2[35, ]))
  df_MD_Unst = t(as.matrix(result2[36, ]))
  df_MD_Unst_New = t(as.matrix(result2[37, ]))
  
  #For each, I am interested in 3 types of df: a) n-parameters, b) the Pan and Wall approach, and
  #c) Wang & Long and Westgate & Burchette approaches. This next section calculated coverage probabilities.
  
  cover0a = cover1a = cover2a = matrix(0, 1, 18)
  cover0b = cover1b = cover2b = matrix(0, 1, 18)
  cover0c = cover1c = cover2c = matrix(0, 1, 4)
  
  #For beta0:
  #a) df = n-parameters
  if(((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 4]) <= Beta_true[1, 1]) &
     ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 4]) >= Beta_true[1, 1])) cover0a[1, 1]=1
  if (((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 5]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 5]) >= Beta_true[1, 1])) cover0a[1, 2]=1
  if (((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 7]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 7]) >= Beta_true[1, 1])) cover0a[1, 3]=1
  if (((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 9]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 9]) >= Beta_true[1, 1])) cover0a[1, 4]=1
  if (((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 10]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 10]) >= Beta_true[1, 1])) cover0a[1, 5]=1
  if (((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 11]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 11]) >= Beta_true[1, 1])) cover0a[1, 6]=1
  if (((result2a[1, 1] - qt(.975, (n - parameters)) * result2a[1, 12]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (n - parameters)) * result2a[1, 12]) >= Beta_true[1, 1])) cover0a[1, 7]=1
  
  #b) df = Pan & Wall
  if(((result2a[1,1] - qt(.975, (result2[27,1])) * result2a[1,4]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[27,1])) * result2a[1,4]) >= Beta_true[1,1])) cover0b[1,1]=1
  if(((result2a[1, 1] - qt(.975, (result2[28, 1])) * result2a[1, 5]) <= Beta_true[1, 1]) &
     ((result2a[1, 1] + qt(.975, (result2[28, 1])) * result2a[1, 5]) >= Beta_true[1, 1])) cover0b[1, 2]=1
  if (((result2a[1, 1] - qt(.975, (result2[30, 1])) * result2a[1, 7]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (result2[30, 1])) * result2a[1, 7]) >= Beta_true[1, 1])) cover0b[1, 3]=1
  if (((result2a[1, 1] - qt(.975, (result2[32, 1])) * result2a[1, 9]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (result2[32, 1])) * result2a[1, 9]) >= Beta_true[1, 1])) cover0b[1, 4]=1
  if (((result2a[1, 1] - qt(.975, (result2[33, 1])) * result2a[1, 10]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (result2[33, 1])) * result2a[1, 10]) >= Beta_true[1, 1])) cover0b[1, 5]=1
  if (((result2a[1, 1] - qt(.975, (result2[34, 1])) * result2a[1, 11]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (result2[34, 1])) * result2a[1, 11]) >= Beta_true[1, 1])) cover0b[1, 6]=1
  if (((result2a[1, 1] - qt(.975, (result2[35, 1])) * result2a[1, 12]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, (result2[35, 1])) * result2a[1, 12]) >= Beta_true[1, 1])) cover0b[1, 7]=1
  
  #c) df = Wang & Long or Westgate & Burchett
  if (((result2a[1, 1] - qt(.975, result2[36, 1]) * result2a[1, 5]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, result2[36, 1]) * result2a[1, 5]) >= Beta_true[1, 1])) cover0c[1, 1] = 1
  if (((result2a[1, 1] - qt(.975, result2[37, 1]) * result2a[1, 5]) <= Beta_true[1, 1]) &
      ((result2a[1, 1] + qt(.975, result2[37, 1]) * result2a[1, 5]) >= Beta_true[1, 1])) cover0c[1, 2] = 1
  
  
  #For beta1:
  #a) df = n-parameters
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 4]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 4]) >= Beta_true[2, 1])) cover1a[1, 1] = 1
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 5]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 5]) >= Beta_true[2, 1])) cover1a[1, 2] = 1
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 7]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 7]) >= Beta_true[2, 1])) cover1a[1, 3] = 1
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 9]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 9]) >= Beta_true[2, 1])) cover1a[1, 4] = 1
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 10]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 10]) >= Beta_true[2, 1])) cover1a[1, 5] = 1
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 11]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 11]) >= Beta_true[2, 1])) cover1a[1, 6] = 1
  if (((result2b[1, 1] - qt(.975, (n - parameters)) * result2b[1, 12]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (n - parameters)) * result2b[1, 12]) >= Beta_true[2, 1])) cover1a[1, 7] = 1
  
  #b) df = Pan & Wall
  if (((result2b[1, 1] - qt(.975, (result2[27, 2])) * result2b[1, 4]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[27, 2])) * result2b[1, 4]) >= Beta_true[2, 1])) cover1b[1, 1] = 1
  if (((result2b[1, 1] - qt(.975, (result2[28, 2])) * result2b[1, 5]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[28, 2])) * result2b[1, 5]) >= Beta_true[2, 1])) cover1b[1, 2] = 1
  if (((result2b[1, 1] - qt(.975, (result2[30, 2])) * result2b[1, 7]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[30, 2])) * result2b[1, 7]) >= Beta_true[2, 1])) cover1b[1, 3] = 1
  if (((result2b[1, 1] - qt(.975, (result2[32, 2])) * result2b[1, 9]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[32, 2])) * result2b[1, 9]) >= Beta_true[2, 1])) cover1b[1, 4] = 1
  if (((result2b[1, 1] - qt(.975, (result2[33, 2])) * result2b[1, 10]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[33, 2])) * result2b[1, 10]) >= Beta_true[2, 1])) cover1b[1, 5] = 1
  if (((result2b[1, 1] - qt(.975, (result2[34, 2])) * result2b[1, 11]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[34, 2])) * result2b[1, 11]) >= Beta_true[2, 1])) cover1b[1, 6] = 1
  if (((result2b[1, 1] - qt(.975, (result2[35, 2])) * result2b[1, 12]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, (result2[35, 2])) * result2b[1, 12]) >= Beta_true[2, 1])) cover1b[1, 7] = 1
  
  #c) df = Wang & Long or Westgate & Burchett
  if (((result2b[1, 1] - qt(.975, result2[36, 2]) * result2b[1, 5]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, result2[36, 2]) * result2b[1, 5]) >= Beta_true[2, 1])) cover1c[1, 1] = 1
  if (((result2b[1, 1] - qt(.975, result2[37, 2]) * result2b[1, 5]) <= Beta_true[2, 1]) &
      ((result2b[1, 1] + qt(.975, result2[37, 2]) * result2b[1, 5]) >= Beta_true[2, 1])) cover1c[1, 2] = 1
  
  
  no_penalty_results = cbind(cover0a,cover0b,cover0c,cover1a,cover1b,cover1c,cover2a,cover2b,cover2c)
  
  result_2 = cbind(result2a,result2b,result2c,ICC_est,var_est,df_KC,df_MD,df_MD_Unst,df_MD_Unst_New,cover2c,no_penalty_results)
  RESULT2 = rbind(RESULT2,result_2)
  
  print(j)
  j
} #end of the simulations 


RESULT2 = as.matrix(RESULT2)



#Results for Exchangeable working correlation structure
summary2 = matrix(0, 90, 6)
summary2[1:23, 1] = as.matrix(apply(RESULT2[, 1:23], 2, mean))          #Empirical means of the estimate for beta0 and corresponding SE estimates
summary2[1:23, 2] = sqrt(as.matrix(apply(RESULT2[, 1:23], 2, var)))     #Empirical standard deviations of the estimate for beta0 and corresponding SE estimates
summary2[1:23, 3] = as.matrix(apply(RESULT2[, 24:46], 2, mean))         #For beta1
summary2[1:23, 4] = sqrt(as.matrix(apply(RESULT2[, 24:46], 2, var)))    #For beta1

summary2[26:43, 1] = 1 - as.matrix(apply(RESULT2[, 84:101], 2, mean))        #Empirical CP of beta0 corresponding to N-p df and the 4 SE estimates: KC with my correction, MD                                                                                                                                              #with my correction, KC with my correction and assuming common cov., MD with my correction and assuming common cov.
summary2[44:61, 1] = 1 - as.matrix(apply(RESULT2[, 102:119], 2, mean))        #Empirical CP of beta0 corresponding to Pan and Wall etc. and corresponding SE estimates
summary2[62:65, 1] = 1 - as.matrix(apply(RESULT2[, 120:123], 2, mean))        #Empirical CP of beta0 corresponding to Westgate & Burchette correction

summary2[26:43, 3] = 1 - as.matrix(apply(RESULT2[, 124:141], 2, mean))        #Empirical CP of beta1 corresponding to N-p df and the 4 SE estimates
summary2[44:61, 3] = 1 - as.matrix(apply(RESULT2[, 142:159], 2, mean))        #Empirical CP of beta1 corresponding to Pan and Wall (or Wang and Long) df and the 4 SE estimates
summary2[62:65, 3] = 1 - as.matrix(apply(RESULT2[, 160:163], 2, mean))        #Empirical CP of beta1 corresponding to between-within df and the 4 SE estimates


Zero = matrix(0, 90, 1)

Summary = cbind(summary2, Zero)

write.table(Summary, file='Sim_Results.txt', sep="\t", col.names=TRUE)  