#This code will produce the simulation results for continuous Scenario 4 with unequal temporal spacing
#reflected in outputs Figures 82-85 of supplementary materials.

RESULT1 = NULL
RESULT2 = NULL
RESULT3 = NULL

library(mvtnorm)
library(matrixcalc)


simulations = 1000
np = parameters = 4

n = 10                             #Number of independent subjects: 10, 20, 30, 40
ni_max=sizes_max=5

Cluster_Size = matrix(ni_max, n, 1)

Corr_parm = 0.2                     #This is the correlation parameter for true Exchangeable or AR-1
Corr_structure = "exchangeable"             #This is the true correlation structure.


Beta_true = matrix(0, parameters, 1)
Beta_true[1, 1] = 0    #Intercept
Beta_true[2, 1] = 0    #Time effect
Beta_true[3, 1] = 0    #Time^2 effect
Beta_true[4, 1] = 0    #Time independent covariate

#Start of the simulations
for (j in 1:simulations)
{
  X = NULL
  time_tot = NULL
  for (i in 1:n)
  {
    times_observed = matrix(0, ni_max, 1)
    if (ni_max == 5) {
      a = c(.5, 1, 1.5, 2, 2.5, 3, 3.5, 4)
      times_after = sample(a, 4)
      times_observed[2:5, 1] = sort(times_after)
    }
    if (ni_max == 10) {
      a = c(.5, 1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5, 7, 7.5, 8)
      times_after = sample(a, 9)
      times_observed[2:10, 1] = sort(times_after)
    }
    
    a_old = sample(1:Cluster_Size[i, 1], Cluster_Size[i, 1])
    time_orig = as.matrix(sort(a_old, decreasing = FALSE))
    time  = time_orig + (ni_max - Cluster_Size[i, 1])   # Observation number for functions matrix
    
    timesq = times_observed ^ 2
    timeind = as.matrix(rnorm(Cluster_Size[i, 1], mean = times_observed, sd =
                                1))
    
    variance = matrix(100, length(time), 1)                            #Variances for the outcomes
    
    Design = matrix(1, Cluster_Size[i, 1], (parameters + 1))             #Implicitly gives values of 1 in the design matrix for the intercept, beta
    Design[, 2] = times_observed
    Design[, 3] = timesq
    Design[, 4] = timeind
    Design[, 5] = i                                                  #Subject ID
    
    
    #Setting up the true correlation matrix
    if (Corr_structure == "AR-1")
    {
      Corr = diag(1, Cluster_Size[i, 1])
      for (k in 1:(Cluster_Size[i, 1] - 1))
        for (l in (k + 1):Cluster_Size[i, 1])
          Corr[k, l] = Corr[l, k] = Corr_parm ^ (abs(Design[k, 2] - Design[l, 2]))   #Note that the correlation matrix is the covariance matrix too
    }
    
    if (Corr_structure == "exchangeable")
      Corr = matrix(Corr_parm, Cluster_Size[i, 1], Cluster_Size[i, 1]) + diag((1 -
                                                                                 Corr_parm), Cluster_Size[i, 1])
    
    
    V = diag(as.vector(variance), Cluster_Size[i, 1], Cluster_Size[i, 1]) #True marginal variances
    Cov = sqrt(V) %*% Corr %*% sqrt(V)                                  #True covariance structure
    
    Errors = t(as.matrix(rmvnorm(1, sigma = Cov)))                    #Randomly generating the true errors
    
    Y = Design[, 1:4] %*% Beta_true + Errors                            #Randomly generated outcomes
    
    Design = cbind(Design, Y)
    X = rbind(X, Design)
    time_tot = rbind(time_tot, time_orig)
    
  }
  
  
  #Now setting up the input (for GEE function) data matrices
  
  id = as.matrix(X[, 5])   #Matrix of subject IDs
  y = as.matrix(X[, 6])    #Matrix of outcome values
  x = cbind(X[, 2], X[, 3], X[, 4])  #Matrix of covariate values
  time = as.matrix(time_tot) #Matrix of time points
  
  
  ##############################################################
  #GEE results using the working exchangeable correlation structure
  ##############################################################
  result2 = GEE(id, y, x, "gaussian", "exchangeable", NULL, 200, 0.00000001, time)
  result2a = t(as.matrix(result2[1:23, 1]))  #Results with respect to beta0
  result2b = t(as.matrix(result2[1:23, 2]))  #Results with respect to beta1
  result2c = t(as.matrix(result2[1:23, 3]))  #Results with respect to beta2
  result2d = t(as.matrix(result2[1:23, 4]))  #Results with respect to beta2
  ICC_est = result2[25, 1]
  var_est = result2[26, 2]
  
  df_KC = t(as.matrix(result2[27, ]))
  df_MD = t(as.matrix(result2[28, ]))
  df_AVG = t(as.matrix(result2[30, ]))
  df_FG = t(as.matrix(result2[32, ]))
  df_GSH = t(as.matrix(result2[33, ]))
  df_FZZ = t(as.matrix(result2[34, ]))
  df_MBN = t(as.matrix(result2[35, ]))
  df_MD_Unst = t(as.matrix(result2[36, ]))
  df_MD_Unst_New = t(as.matrix(result2[37, ]))
  
  #For each, I am interested in 3 types of df: a) n-parameters, b) the Pan and Wall approach, and 
  #c) Wang & Long and Westgate & Burchette approaches. This next section calculated coverage probabilities.
  
  cover0a = cover1a = cover2a = cover3a = matrix(0, 1, 18)
  cover0b = cover1b = cover2b = cover3b = matrix(0, 1, 18)
  cover0c = cover1c = cover2c = cover3c = matrix(0, 1, 4)
  
  #For beta0:
  #a) df = n-parameters
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,4]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,4]) >= Beta_true[1,1])) cover0a[1,1]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,5]) >= Beta_true[1,1])) cover0a[1,2]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,7]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,7]) >= Beta_true[1,1])) cover0a[1,3]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,9]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,9]) >= Beta_true[1,1])) cover0a[1,4]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,10]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,10]) >= Beta_true[1,1])) cover0a[1,5]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,11]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,11]) >= Beta_true[1,1])) cover0a[1,6]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,12]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,12]) >= Beta_true[1,1])) cover0a[1,7]=1
  
  #b) df = Pan & Wall 
  if(((result2a[1,1] - qt(.975, (result2[27,1])) * result2a[1,4]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[27,1])) * result2a[1,4]) >= Beta_true[1,1])) cover0b[1,1]=1
  if(((result2a[1,1] - qt(.975, (result2[28,1])) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[28,1])) * result2a[1,5]) >= Beta_true[1,1])) cover0b[1,2]=1
  if(((result2a[1,1] - qt(.975, (result2[30,1])) * result2a[1,7]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[30,1])) * result2a[1,7]) >= Beta_true[1,1])) cover0b[1,3]=1
  if(((result2a[1,1] - qt(.975, (result2[32,1])) * result2a[1,9]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[32,1])) * result2a[1,9]) >= Beta_true[1,1])) cover0b[1,4]=1
  if(((result2a[1,1] - qt(.975, (result2[33,1])) * result2a[1,10]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[33,1])) * result2a[1,10]) >= Beta_true[1,1])) cover0b[1,5]=1
  if(((result2a[1,1] - qt(.975, (result2[34,1])) * result2a[1,11]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[34,1])) * result2a[1,11]) >= Beta_true[1,1])) cover0b[1,6]=1
  if(((result2a[1,1] - qt(.975, (result2[35,1])) * result2a[1,12]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[35,1])) * result2a[1,12]) >= Beta_true[1,1])) cover0b[1,7]=1
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2a[1,1] - qt(.975,result2[36,1]) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975,result2[36,1]) * result2a[1,5]) >= Beta_true[1,1])) cover0c[1,1]=1
  if(((result2a[1,1] - qt(.975,result2[37,1]) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975,result2[37,1]) * result2a[1,5]) >= Beta_true[1,1])) cover0c[1,2]=1
  
  
  
  #For beta1:
  #a) df = n-parameters
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,4]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,4]) >= Beta_true[2,1])) cover1a[1,1]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,5]) >= Beta_true[2,1])) cover1a[1,2]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,7]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,7]) >= Beta_true[2,1])) cover1a[1,3]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,9]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,9]) >= Beta_true[2,1])) cover1a[1,4]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,10]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,10]) >= Beta_true[2,1])) cover1a[1,5]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,11]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,11]) >= Beta_true[2,1])) cover1a[1,6]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,12]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,12]) >= Beta_true[2,1])) cover1a[1,7]=1
  
  
  #b) df = Pan & Wall
  if(((result2b[1,1] - qt(.975, (result2[27,2])) * result2b[1,4]) <= Beta_true[2,1]) &
     ((result2b[1,1] + qt(.975, (result2[27,2])) * result2b[1,4]) >= Beta_true[2,1])) cover1b[1,1]=1
  if(((result2b[1,1] - qt(.975, (result2[28,2])) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[28,2])) * result2b[1,5]) >= Beta_true[2,1])) cover1b[1,2]=1
  if(((result2b[1,1] - qt(.975, (result2[30,2])) * result2b[1,7]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[30,2])) * result2b[1,7]) >= Beta_true[2,1])) cover1b[1,3]=1
  if(((result2b[1,1] - qt(.975, (result2[32,2])) * result2b[1,9]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[32,2])) * result2b[1,9]) >= Beta_true[2,1])) cover1b[1,4]=1
  if(((result2b[1,1] - qt(.975, (result2[33,2])) * result2b[1,10]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[33,2])) * result2b[1,10]) >= Beta_true[2,1])) cover1b[1,5]=1
  if(((result2b[1,1] - qt(.975, (result2[34,2])) * result2b[1,11]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[34,2])) * result2b[1,11]) >= Beta_true[2,1])) cover1b[1,6]=1
  if(((result2b[1,1] - qt(.975, (result2[35,2])) * result2b[1,12]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[35,2])) * result2b[1,12]) >= Beta_true[2,1])) cover1b[1,7]=1
  
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2b[1,1] - qt(.975,result2[36,2]) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975,result2[36,2]) * result2b[1,5]) >= Beta_true[2,1])) cover1c[1,1]=1
  if(((result2b[1,1] - qt(.975,result2[37,2]) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975,result2[37,2]) * result2b[1,5]) >= Beta_true[2,1])) cover1c[1,2]=1
  
  #For beta2:
  #a) df = n-parameters
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,4]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,4]) >= Beta_true[3,1])) cover2a[1,1]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,5]) >= Beta_true[3,1])) cover2a[1,2]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,7]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,7]) >= Beta_true[3,1])) cover2a[1,3]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,9]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,9]) >= Beta_true[3,1])) cover2a[1,4]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,10]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,10]) >= Beta_true[3,1])) cover2a[1,5]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,11]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,11]) >= Beta_true[3,1])) cover2a[1,6]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,12]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,12]) >= Beta_true[3,1])) cover2a[1,7]=1
  
  
  #b) df = Pan & Wall
  if(((result2c[1,1] - qt(.975, (result2[27,3])) * result2c[1,4]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[27,3])) * result2c[1,4]) >= Beta_true[3,1])) cover2b[1,1]=1
  if(((result2c[1,1] - qt(.975, (result2[28,3])) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[28,3])) * result2c[1,5]) >= Beta_true[3,1])) cover2b[1,2]=1
  if(((result2c[1,1] - qt(.975, (result2[30,3])) * result2c[1,7]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[30,3])) * result2c[1,7]) >= Beta_true[3,1])) cover2b[1,3]=1
  if(((result2c[1,1] - qt(.975, (result2[32,3])) * result2c[1,9]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[32,3])) * result2c[1,9]) >= Beta_true[3,1])) cover2b[1,4]=1
  if(((result2c[1,1] - qt(.975, (result2[33,3])) * result2c[1,10]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[33,3])) * result2c[1,10]) >= Beta_true[3,1])) cover2b[1,5]=1
  if(((result2c[1,1] - qt(.975, (result2[34,3])) * result2c[1,11]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[34,3])) * result2c[1,11]) >= Beta_true[3,1])) cover2b[1,6]=1
  if(((result2c[1,1] - qt(.975, (result2[35,3])) * result2c[1,12]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[35,3])) * result2c[1,12]) >= Beta_true[3,1])) cover2b[1,7]=1
  
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2c[1,1] - qt(.975,result2[36,3]) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975,result2[36,3]) * result2c[1,5]) >= Beta_true[3,1])) cover2c[1,1]=1
  if(((result2c[1,1] - qt(.975,result2[37,3]) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975,result2[37,3]) * result2c[1,5]) >= Beta_true[3,1])) cover2c[1,2]=1
  
  
  #For beta3:
  #a) df = n-parameters
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,4]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,4]) >= Beta_true[4,1])) cover3a[1,1]=1
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,5]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,5]) >= Beta_true[4,1])) cover3a[1,2]=1
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,7]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,7]) >= Beta_true[4,1])) cover3a[1,4]=1
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,9]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,9]) >= Beta_true[4,1])) cover3a[1,4]=1
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,10]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,10]) >= Beta_true[4,1])) cover3a[1,5]=1
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,11]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,11]) >= Beta_true[4,1])) cover3a[1,6]=1
  if(((result2d[1,1] - qt(.975, (n - parameters)) * result2d[1,12]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (n - parameters)) * result2d[1,12]) >= Beta_true[4,1])) cover3a[1,7]=1
  
  
  #b) df = Pan & Wall
  if(((result2d[1,1] - qt(.975, (result2[27,4])) * result2d[1,4]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[27,4])) * result2d[1,4]) >= Beta_true[4,1])) cover3b[1,1]=1
  if(((result2d[1,1] - qt(.975, (result2[28,4])) * result2d[1,5]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[28,4])) * result2d[1,5]) >= Beta_true[4,1])) cover3b[1,2]=1
  if(((result2d[1,1] - qt(.975, (result2[30,4])) * result2d[1,7]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[30,4])) * result2d[1,7]) >= Beta_true[4,1])) cover3b[1,4]=1
  if(((result2d[1,1] - qt(.975, (result2[32,4])) * result2d[1,9]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[32,4])) * result2d[1,9]) >= Beta_true[4,1])) cover3b[1,4]=1
  if(((result2d[1,1] - qt(.975, (result2[33,4])) * result2d[1,10]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[33,4])) * result2d[1,10]) >= Beta_true[4,1])) cover3b[1,5]=1
  if(((result2d[1,1] - qt(.975, (result2[34,4])) * result2d[1,11]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[34,4])) * result2d[1,11]) >= Beta_true[4,1])) cover3b[1,6]=1
  if(((result2d[1,1] - qt(.975, (result2[35,4])) * result2d[1,12]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975, (result2[35,4])) * result2d[1,12]) >= Beta_true[4,1])) cover3b[1,7]=1
  
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2d[1,1] - qt(.975,result2[36,4]) * result2d[1,5]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975,result2[36,4]) * result2d[1,5]) >= Beta_true[4,1])) cover3c[1,1]=1
  if(((result2d[1,1] - qt(.975,result2[37,4]) * result2d[1,5]) <= Beta_true[4,1]) & 
     ((result2d[1,1] + qt(.975,result2[37,4]) * result2d[1,5]) >= Beta_true[4,1])) cover3c[1,2]=1
  
  
  no_penalty_results = cbind(cover0a,cover0b,cover0c,cover1a,cover1b,cover1c,cover2a,cover2b,cover2c,cover3a,cover3b,cover3c)
  result_2 = cbind(result2a,result2b,result2c,result2d,ICC_est,var_est,df_KC,df_MD,df_MD_Unst,df_MD_Unst_New,no_penalty_results)
  RESULT2 = rbind(RESULT2,result_2)
  
  print(j)
  
} #end of the simulations 




RESULT2=as.matrix(RESULT2)


#Results for Exchangeable
summary2 = matrix(0, 90, 8)
summary2[1:23, 1] = as.matrix(apply(RESULT2[, 1:23], 2, mean))          #Empirical means of the estimate for beta0 and corresponding SE estimates
summary2[1:23, 2] = sqrt(as.matrix(apply(RESULT2[, 1:23], 2, var)))     #Empirical standard deviations of the estimate for beta0 and corresponding SE estimates
summary2[1:23, 3] = as.matrix(apply(RESULT2[, 24:46], 2, mean))         #For beta1
summary2[1:23, 4] = sqrt(as.matrix(apply(RESULT2[, 24:46], 2, var)))    #For beta1
summary2[1:23, 5] = as.matrix(apply(RESULT2[, 47:69], 2, mean))         #For beta2
summary2[1:23, 6] = sqrt(as.matrix(apply(RESULT2[, 47:69], 2, var)))    #For beta2
summary2[1:23, 7] = as.matrix(apply(RESULT2[, 70:92], 2, mean))         #For beta3
summary2[1:23, 8] = sqrt(as.matrix(apply(RESULT2[, 70:92], 2, var)))    #For beta3

summary2[26:43, 1] = 1 - as.matrix(apply(RESULT2[, 111:128], 2, mean))        #Empirical CP of beta0 corresponding to N-p df 
summary2[44:61, 1] = 1 - as.matrix(apply(RESULT2[, 129:146], 2, mean))        #Empirical CP of beta0 corresponding to Pan and Wall df
summary2[62:65, 1] = 1 - as.matrix(apply(RESULT2[, 147:150], 2, mean))        #Empirical CP of beta0 corresponding to Wang & Long and Westgate and Burchett df

summary2[26:43, 3] = 1 - as.matrix(apply(RESULT2[, 151:168], 2, mean))        #Empirical CP of beta1 corresponding to N-p df 
summary2[44:61, 3] = 1 - as.matrix(apply(RESULT2[, 169:186], 2, mean))        #Empirical CP of beta1 corresponding to Pan and Wall df
summary2[62:65, 3] = 1 - as.matrix(apply(RESULT2[, 187:190], 2, mean))        #Empirical CP of beta1 corresponding to Wang & Long and Westgate and Burchett df

summary2[26:43, 5] = 1 - as.matrix(apply(RESULT2[, 191:208], 2, mean))        #Empirical CP of beta2 corresponding to N-p df 
summary2[44:61, 5] = 1 - as.matrix(apply(RESULT2[, 209:226], 2, mean))        #Empirical CP of beta2 corresponding to Pan and Wall df
summary2[62:65, 5] = 1 - as.matrix(apply(RESULT2[, 227:230], 2, mean))        #Empirical CP of beta2 corresponding to Wang & Long and Westgate and Burchett df

summary2[26:43, 7] = 1 - as.matrix(apply(RESULT2[, 231:248], 2, mean))        #Empirical CP of beta3 corresponding to N-p df
summary2[44:61, 7] = 1 - as.matrix(apply(RESULT2[, 249:266], 2, mean))        #Empirical CP of beta3 corresponding to Pan and Wall df
summary2[62:65, 7] = 1 - as.matrix(apply(RESULT2[, 267:270], 2, mean))        #Empirical CP of beta3 corresponding to Wang & Long and Westgate and Burchett df



Zero = matrix(0, 90, 1)

Summary = cbind(summary2, Zero)

write.table(Summary, file='Sim_Results.txt', sep="\t", col.names=TRUE)  
