#This code will produce the simulation results for Poisson Scenario 3 reflected in outputs  
#Figure 16 of main manuscript and Figures 32-33 of Supplementary Materials.

library(matrixcalc)

RESULT1 = NULL
RESULT2 = NULL
RESULT3 = NULL

simulations = 1000

np = parameters = 3
n = 10                             #Number of independent subjects: 10, 20, 30, 40
ni = 5                             #Responses: 5, 10


#Per WL, correlation paramaeter for Poisson outcomes is ~0.3
Corr_parm = 0.3
Corr_structure = "exchangeable"     #This is the true correlation structure.


Beta_true = matrix(0, parameters, 1)
Beta_true[1, 1] = 0      #Intercept
Beta_true[2, 1] = 0      #Time effect
Beta_true[3, 1] = 0      #Interaction effect


#Now for simulations

for(j in 1:simulations)
{

  Cluster_Size = matrix(ni, n, 1) #Matrix containing the number of observations from each subject
  subjleff = as.matrix(rnorm(n, mean = 0, sd = sqrt(0.25))) #subject level random effects, bi
  
  X = NULL
  Y = NULL
  time_tot = NULL
  means_tot = NULL
  Data = matrix(0, ni, 1)
  
for(i in 1:n)
{

  a = sample(1:ni, Cluster_Size[i, 1])
  time = as.matrix(sort(a, decreasing = FALSE))                     #Observed time points in order
  treat = matrix(0, ni, 1)
  if (i > (n / 2))
    treat = matrix(1, ni, 1)
  
  interaction = treat * time
  
  Design = matrix(1, Cluster_Size[i, 1], (parameters + 1))             #Implicitly gives values of 1 in the design matrix for the intercept, beta
  Design[, 2] = time
  Design[, 3] = interaction                                        #Observed time points in order
  Design[, 4] = i                                                  #Subject ID
  
  #Here are the linear predictors
  lin_pred = Design[, 1:3] %*% Beta_true + subjleff[i, 1]
  
  #Here are the resulting poisson means
  means = exp(lin_pred)
  
  time_tot = rbind(time_tot, time)
  X = rbind(X, Design)
  
  
  Data[, 1] = rpois(ni, means)           #generating cluster-specific success counts
  Y = rbind(Data, Y)
}  

  #Now setting up the input (for GEE function) data matrices
  
  id = as.matrix(X[, 4])   #Matrix of subject IDs
  y = as.matrix(Y)    #Matrix of outcome values
  x = as.matrix(X[, 2:3])  #Matrix of covariate values
  time = as.matrix(time_tot) #Matrix of time points
  
  
  ##############################################################
  #GEE results using the working exchangeable correlation structure
  ##############################################################
  result2 = GEE(id, y, x, "poisson", "exchangeable", NULL, 200, 0.00000001, time)
  result2a = t(as.matrix(result2[1:23, 1]))  #Results with respect to beta0
  result2b = t(as.matrix(result2[1:23, 2]))  #Results with respect to beta1
  result2c = t(as.matrix(result2[1:23, 3]))  #Results with respect to beta2
  ICC_est = result2[25, 1]
  var_est = result2[26, 2]
  
  df_KC = t(as.matrix(result2[27, ]))
  df_MD = t(as.matrix(result2[28, ]))
  df_AVG = t(as.matrix(result2[30, ]))
  df_FG = t(as.matrix(result2[32, ]))
  df_GSH = t(as.matrix(result2[33, ]))
  df_FZZ = t(as.matrix(result2[34, ]))
  df_MBN = t(as.matrix(result2[35, ]))
  df_MD_Unst = t(as.matrix(result2[36, ]))
  df_MD_Unst_New = t(as.matrix(result2[37, ]))
  
  #For each, I am interested in 3 types of df: a) n-parameters, b) the Pan and Wall approach, and 
  #c) Wang & Long and Westgate & Burchette approaches. This next section calculated coverage probabilities.
  
  cover0a = cover1a = cover2a = matrix(0, 1, 18)
  cover0b = cover1b = cover2b = matrix(0, 1, 18)
  cover0c = cover1c = cover2c = matrix(0, 1, 4)
  
  #For beta0:
  #a) df = n-parameters
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,4]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,4]) >= Beta_true[1,1])) cover0a[1,1]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,5]) >= Beta_true[1,1])) cover0a[1,2]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,7]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,7]) >= Beta_true[1,1])) cover0a[1,3]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,9]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,9]) >= Beta_true[1,1])) cover0a[1,4]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,10]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,10]) >= Beta_true[1,1])) cover0a[1,5]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,11]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,11]) >= Beta_true[1,1])) cover0a[1,6]=1
  if(((result2a[1,1] - qt(.975, (n - parameters)) * result2a[1,12]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (n - parameters)) * result2a[1,12]) >= Beta_true[1,1])) cover0a[1,7]=1
  
  #b) df = Pan & Wall 
  if(((result2a[1,1] - qt(.975, (result2[27,1])) * result2a[1,4]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[27,1])) * result2a[1,4]) >= Beta_true[1,1])) cover0b[1,1]=1
  if(((result2a[1,1] - qt(.975, (result2[28,1])) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[28,1])) * result2a[1,5]) >= Beta_true[1,1])) cover0b[1,2]=1
  if(((result2a[1,1] - qt(.975, (result2[30,1])) * result2a[1,7]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[30,1])) * result2a[1,7]) >= Beta_true[1,1])) cover0b[1,3]=1
  if(((result2a[1,1] - qt(.975, (result2[32,1])) * result2a[1,9]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[32,1])) * result2a[1,9]) >= Beta_true[1,1])) cover0b[1,4]=1
  if(((result2a[1,1] - qt(.975, (result2[33,1])) * result2a[1,10]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[33,1])) * result2a[1,10]) >= Beta_true[1,1])) cover0b[1,5]=1
  if(((result2a[1,1] - qt(.975, (result2[34,1])) * result2a[1,11]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[34,1])) * result2a[1,11]) >= Beta_true[1,1])) cover0b[1,6]=1
  if(((result2a[1,1] - qt(.975, (result2[35,1])) * result2a[1,12]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975, (result2[35,1])) * result2a[1,12]) >= Beta_true[1,1])) cover0b[1,7]=1
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2a[1,1] - qt(.975,result2[36,1]) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975,result2[36,1]) * result2a[1,5]) >= Beta_true[1,1])) cover0c[1,1]=1
  if(((result2a[1,1] - qt(.975,result2[37,1]) * result2a[1,5]) <= Beta_true[1,1]) & 
     ((result2a[1,1] + qt(.975,result2[37,1]) * result2a[1,5]) >= Beta_true[1,1])) cover0c[1,2]=1
  
  
  
  #For beta1:
  #a) df = n-parameters
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,4]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,4]) >= Beta_true[2,1])) cover1a[1,1]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,5]) >= Beta_true[2,1])) cover1a[1,2]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,7]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,7]) >= Beta_true[2,1])) cover1a[1,3]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,9]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,9]) >= Beta_true[2,1])) cover1a[1,4]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,10]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,10]) >= Beta_true[2,1])) cover1a[1,5]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,11]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,11]) >= Beta_true[2,1])) cover1a[1,6]=1
  if(((result2b[1,1] - qt(.975, (n - parameters)) * result2b[1,12]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (n - parameters)) * result2b[1,12]) >= Beta_true[2,1])) cover1a[1,7]=1
  
  
  #b) df = Pan & Wall
  if(((result2b[1,1] - qt(.975, (result2[27,2])) * result2b[1,4]) <= Beta_true[2,1]) &
     ((result2b[1,1] + qt(.975, (result2[27,2])) * result2b[1,4]) >= Beta_true[2,1])) cover1b[1,1]=1
  if(((result2b[1,1] - qt(.975, (result2[28,2])) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[28,2])) * result2b[1,5]) >= Beta_true[2,1])) cover1b[1,2]=1
  if(((result2b[1,1] - qt(.975, (result2[30,2])) * result2b[1,7]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[30,2])) * result2b[1,7]) >= Beta_true[2,1])) cover1b[1,3]=1
  if(((result2b[1,1] - qt(.975, (result2[32,2])) * result2b[1,9]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[32,2])) * result2b[1,9]) >= Beta_true[2,1])) cover1b[1,4]=1
  if(((result2b[1,1] - qt(.975, (result2[33,2])) * result2b[1,10]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[33,2])) * result2b[1,10]) >= Beta_true[2,1])) cover1b[1,5]=1
  if(((result2b[1,1] - qt(.975, (result2[34,2])) * result2b[1,11]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[34,2])) * result2b[1,11]) >= Beta_true[2,1])) cover1b[1,6]=1
  if(((result2b[1,1] - qt(.975, (result2[35,2])) * result2b[1,12]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975, (result2[35,2])) * result2b[1,12]) >= Beta_true[2,1])) cover1b[1,7]=1
  
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2b[1,1] - qt(.975,result2[36,2]) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975,result2[36,2]) * result2b[1,5]) >= Beta_true[2,1])) cover1c[1,1]=1
  if(((result2b[1,1] - qt(.975,result2[37,2]) * result2b[1,5]) <= Beta_true[2,1]) & 
     ((result2b[1,1] + qt(.975,result2[37,2]) * result2b[1,5]) >= Beta_true[2,1])) cover1c[1,2]=1
  
  #For beta2:
  #a) df = n-parameters
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,4]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,4]) >= Beta_true[3,1])) cover2a[1,1]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,5]) >= Beta_true[3,1])) cover2a[1,2]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,7]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,7]) >= Beta_true[3,1])) cover2a[1,3]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,9]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,9]) >= Beta_true[3,1])) cover2a[1,4]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,10]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,10]) >= Beta_true[3,1])) cover2a[1,5]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,11]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,11]) >= Beta_true[3,1])) cover2a[1,6]=1
  if(((result2c[1,1] - qt(.975, (n - parameters)) * result2c[1,12]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (n - parameters)) * result2c[1,12]) >= Beta_true[3,1])) cover2a[1,7]=1
  
  
  #b) df = Pan & Wall
  if(((result2c[1,1] - qt(.975, (result2[27,3])) * result2c[1,4]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[27,3])) * result2c[1,4]) >= Beta_true[3,1])) cover2b[1,1]=1
  if(((result2c[1,1] - qt(.975, (result2[28,3])) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[28,3])) * result2c[1,5]) >= Beta_true[3,1])) cover2b[1,2]=1
  if(((result2c[1,1] - qt(.975, (result2[30,3])) * result2c[1,7]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[30,3])) * result2c[1,7]) >= Beta_true[3,1])) cover2b[1,3]=1
  if(((result2c[1,1] - qt(.975, (result2[32,3])) * result2c[1,9]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[32,3])) * result2c[1,9]) >= Beta_true[3,1])) cover2b[1,4]=1
  if(((result2c[1,1] - qt(.975, (result2[33,3])) * result2c[1,10]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[33,3])) * result2c[1,10]) >= Beta_true[3,1])) cover2b[1,5]=1
  if(((result2c[1,1] - qt(.975, (result2[34,3])) * result2c[1,11]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[34,3])) * result2c[1,11]) >= Beta_true[3,1])) cover2b[1,6]=1
  if(((result2c[1,1] - qt(.975, (result2[35,3])) * result2c[1,12]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975, (result2[35,3])) * result2c[1,12]) >= Beta_true[3,1])) cover2b[1,7]=1
  
  
  #c) df = Wang & Long or Westgate & Burchett
  if(((result2c[1,1] - qt(.975,result2[36,3]) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975,result2[36,3]) * result2c[1,5]) >= Beta_true[3,1])) cover2c[1,1]=1
  if(((result2c[1,1] - qt(.975,result2[37,3]) * result2c[1,5]) <= Beta_true[3,1]) & 
     ((result2c[1,1] + qt(.975,result2[37,3]) * result2c[1,5]) >= Beta_true[3,1])) cover2c[1,2]=1
  
  
  
  no_penalty_results = cbind(cover0a,cover0b,cover0c,cover1a,cover1b,cover1c,cover2a,cover2b,cover2c)
  
  
  result_2 = cbind(result2a,result2b,result2c,ICC_est,var_est,df_KC,df_MD,df_MD_Unst,df_MD_Unst_New,no_penalty_results)
  RESULT2 = rbind(RESULT2,result_2)
  
  
  print(j)
  
} #end of the simulations 




RESULT2=as.matrix(RESULT2)

#Results for Exchangeable working correlation structure
summary2 = matrix(0, 90, 6)
summary2[1:23, 1] = as.matrix(apply(RESULT2[, 1:23], 2, mean))          #Empirical means of the estimate for beta0 and corresponding SE estimates
summary2[1:23, 2] = sqrt(as.matrix(apply(RESULT2[, 1:23], 2, var)))     #Empirical standard deviations of the estimate for beta0 and corresponding SE estimates
summary2[1:23, 3] = as.matrix(apply(RESULT2[, 24:46], 2, mean))         #For beta1
summary2[1:23, 4] = sqrt(as.matrix(apply(RESULT2[, 24:46], 2, var)))    #For beta1
summary2[1:23, 5] = as.matrix(apply(RESULT2[, 47:69], 2, mean))         #For beta2
summary2[1:23, 6] = sqrt(as.matrix(apply(RESULT2[, 47:69], 2, var)))    #For beta2


summary2[26:43, 1] = 1 - as.matrix(apply(RESULT2[, 84:101], 2, mean))        #Empirical Type I error rates of beta0 corresponding to N-p df
summary2[44:61, 1] = 1 - as.matrix(apply(RESULT2[, 102:119], 2, mean))        #Empirical Type I error rates of beta0 corresponding to Pan and Wall df
summary2[62:65, 1] = 1 - as.matrix(apply(RESULT2[, 120:123], 2, mean))        #Empirical Type I error rates of beta0 corresponding to WL/WB df

summary2[26:43, 3] = 1 - as.matrix(apply(RESULT2[, 124:141], 2, mean))        #Empirical Type I error rates of beta1 corresponding to N-p df
summary2[44:61, 3] = 1 - as.matrix(apply(RESULT2[, 142:159], 2, mean))        #Empirical Type I error rates of beta1 corresponding to Pan and Wall df
summary2[62:65, 3] = 1 - as.matrix(apply(RESULT2[, 160:163], 2, mean))        #Empirical Type I error rates of beta1 corresponding to WL/WB df

summary2[26:43, 5] = 1 - as.matrix(apply(RESULT2[, 164:181], 2, mean))        #Empirical Type I error rates of beta2 corresponding to N-p df
summary2[44:61, 5] = 1 - as.matrix(apply(RESULT2[, 182:199], 2, mean))        #Empirical Type I error rates of beta2 corresponding to Pan and Wall df
summary2[62:65, 5] = 1 - as.matrix(apply(RESULT2[, 200:203], 2, mean))        #Empirical Type I error rates of beta2 corresponding to WL/WB df


Zero = matrix(0, 90, 1)

Summary = cbind(summary2, Zero)

write.table(Summary, file='Sim_Results.txt', sep="\t", col.names=TRUE)  
